/*
 * alarms.js
 *
 * Copyright (c) 2004 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the Vana'diel Clock.
 *
 * The Vana'diel Clock is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The Vana'diel Clock is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Vana'diel Clock; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Handles the UI for setting alarms and the like.
 */

var editCommand;
var deleteCommand;
var alarmManager;
var treeAlarmsView;

function VanadielClock_alarmLoad() {
    alarmManager = Components.classes["@xenoveritas.org/vanadiel-alarm-manager;1"].
               getService(Components.interfaces.nsIVanadielAlarmManager);
    localizeTimeFormatStrings();
    treeAlarmsView = new AlarmTreeView(alarmManager);
    document.getElementById("alarm-tree").view = treeAlarmsView;
    editCommand = document.getElementById("cmdAlarmProperties");
    deleteCommand = document.getElementById("cmdAlarmDelete");
    // disable a couple of commands
    checkSelection();
}

function VanadielClock_alarmUnload() {
    treeAlarmsView.destroy();
}

function checkSelection() {
    var count = document.getElementById("alarm-tree").view.selection.count;
    if (count > 0) {
	if (count > 1) {
	    if (!editCommand.hasAttribute("disabled"))
		editCommand.setAttribute("disabled", "true");
	} else {
	    if (editCommand.hasAttribute("disabled"))
		editCommand.removeAttribute("disabled");
	}
	if (deleteCommand.hasAttribute("disabled"))
	    deleteCommand.removeAttribute("disabled");
    } else {
	editCommand.setAttribute("disabled", "true");
	deleteCommand.setAttribute("disabled", "true");
    }
}

function newAlarm() {
    VanadielClock_setAlarm();
}

function editAlarm() {
    var tree = document.getElementById("alarm-tree");
    tree.view.performActionOnRow("vc-edit", tree.view.selection.currentIndex);
}

function deleteAlarm() {
    var tree = document.getElementById("alarm-tree");
    var start = new Object();
    var end = new Object();
    var numRanges = tree.view.selection.getRangeCount();

    // since we're removeing (potentially) a bunch of objects at once,
    // ignore incoming events
    treeAlarmsView.ignoreSyncs = true;
    for (var t = 0; t < numRanges; t++) {
	tree.view.selection.getRangeAt(t, start, end);
	for (var v = start.value; v <= end.value; v++) {
	    tree.view.performActionOnRow("vc-delete", v);
	}
    }
    treeAlarmsView.ignoreSyncs = false;
    treeAlarmsView.syncView();
}

// An alarm object (used to detect changes)
function AlarmView(alarm) {
    this.alarm = alarm;
    this.name = alarm.name;
    this.description = alarm.description;
    this.time = alarm.time;
    this.icon = alarm.nextSmallIcon;
}

AlarmView.prototype = {
    alarm: null,
    name: "",
    description: "",
    time: 0,
    icon: null,

    get id() { return this.alarm.id; },

    equals: function(aView) {
	return this.name == aView.name &&
		this.description == aView.description &&
		this.time == aView.time &&
		this.icon == aView.icon;
    }
};

function sortAlarmViewByName(a, b) {
    return a.name < b.name ? -1 :
	    (a.name > b.name ? 1 : 0);
}

function sortAlarmViewByDescription(a, b) {
    return a.description < b.description ? -1 :
	    (a.description > b.description ? 1 : 0);
}

function sortAlarmViewByTime(a, b) {
    return a.time - b.time;
}

function inverseSort(sortFunc) {
    return function(a, b) {
	return 0 - sortFunc(a, b);
    }
}

// Tree view for the alarms

function AlarmTreeView(manager) {
    this._manager = manager;
    this.syncView();
    this._manager.addVanadielAlarmManagerListener(this);
}

AlarmTreeView.prototype = {
    _manager: null,
    _selection: null,
    _tree: null,
    _view: null,
    _sortMethod: sortAlarmViewByName,
    ignoreSyncs: false,

    get rowCount() { return this._manager.count; },
    get selection() { return this._selection; },
    set selection(value) { this._selection = value; },

    // Remove our listener
    destroy: function() {
	this._manager.removeVanadielAlarmManagerListener(this);
    },

    // Syncs up the view with the current list of alarms
    syncView: function() {
	// used for batch operations
	if (this.ignoreSyncs)
	    return;
	var nView = new Array(this._manager.count);
nViewLoop:
	for (var i = 0; i < nView.length; i++) {
	    nView[i] = new AlarmView(this._manager.getAlarmAt(i));
	}
	// Sort the new view
	if (this._sortMethod != null) {
	    nView.sort(this._sortMethod);
	}
	if (this._view != null) {
	    // start invalidating changes, if any
	    var changes = false;
	    if (this._view.length != nView.length) {
		changes = true;
		this._tree.beginUpdateBatch();
		// alert about the inserted/removed rows
		this._tree.rowCountChanged((this._view.length < nView.length ? this._view.length - 1 : this._view.length), nView.length - this._view.length);
	    }
	    var l = this._view.length > nView.length ? nView.length : this._view.length;
	    var start = -1;
	    for (var i = 0; i < l; i++) {
		if (!this._view[i].equals(nView[i])) {
		    if (start < 0) {
			if (!changes) {
			    changes = true;
			    this._tree.beginUpdateBatch();
			}
			start = i;
		    }
		} else if (start >= 0) {
		    if (i - 1 != start)
			this._tree.invalidateRange(start, i-1);
		    else
			this._tree.invalidateRow(start);
		    start = -1;
		}
	    }
	    if (start >= 0) {
		if (i - 1 != start)
		    this._tree.invalidateRange(start, i-1);
		else
		    this._tree.invalidateRow(start);
	    }
	    if (changes) {
		this._tree.endUpdateBatch();
	    }
	}
	this._view = nView;
    },

    canDrop: function(index, orientation) {
	return false;
    },

    cycleCell: function(row, col) { },
    cycleHeader: function(col) {
	var id = col.id;
	if (id == "vcatName") {
	    this._sortMethod = sortAlarmViewByName;
	} else if (id == "vcatDesc") {
	    this._sortMethod = sortAlarmViewByDescription;
	} else if (id == "vcatAt") {
	    this._sortMethod = sortAlarmViewByTime;
	}
	var dir = "ascending";
	if (col.element.getAttribute("sortDirection") == "ascending") {
	    // invert the sort
	    dir = "descending";
	    this._sortMethod = inverseSort(this._sortMethod);
	}
	col.element.setAttribute("sortDirection", dir);
	// set all other columns to "natural"
	var cols = col.columns;
	for (var i = 0; i < cols.count; i++) {
	    var c = cols.getColumnAt(i);
	    if (c != col)
		c.element.setAttribute("sortDirection", "natural");
	}
	this.syncView();
    },

    drop: function(row, orientation) { },
    getCellProperties: function(row, col, properties) { },

    // Gets the cell text
    getCellText: function(row, col) {
	var id = col.id;
	if (id == "vcatName") {
	    return this._view[row].name;
	} else if (id == "vcatDesc") {
	    return this._view[row].description;
	} else if (id == "vcatAt") {
	    var time = VanadielClock_convertToEarthDate(this._view[row].time);
	    return formatEarthDate(time) + " " + formatEarthTime(time, false);
	}
	return "";
    },

    getCellValue: function(row, col) {
	return "";
    },

    getColumnProperties: function(col, properties) { },
    getImageSrc: function(row, col) {
	// TODO: Attempt to guess platform MIME type
	if (col.id == "vcatName") {
	    return this._view[row].icon;
	}
	return null;
    },
    getLevel: function(index) { return 0; },
    getParentIndex: function(rowIndex) { return -1; },
    getProgressMode: function(row, col) { return Components.interfaces.nsITreeView.PROGRESS_NORMAL; },
    getRowProperties: function(index, properties) { },
    hasNextSibling: function(rowIndex, afterIndex) { return afterIndex < this._manager.count; },
    isContainer: function(index) { return false; },
    isContainerEmpty: function(index) { return true; },
    isContainerOpen: function(index) { return false; },
    isEditable: function(row, col) { return false; },
    isSeparator: function(index) { return false; },
    isSorted: function() { return true; },
    performAction: function(action) { },
    performActionOnCell: function(action, row, col) { },
    performActionOnRow: function(action, row) {
	if (action == "vc-edit") {
	    window.openDialog("chrome://vanadielclock/content/alarm-properties.xul", "_blank", "chrome", this._view[row].id);
	} else if (action == "vc-delete") {
	    this._manager.removeAlarm(this._view[row].alarm);
	}
    },
    selectionChanged: function() { },
    setCellText: function(row, col, value) {
	throw Components.results.NS_ERROR_FAILURE;
    },
    setCellValue: function(row, col, value) {
	throw Components.results.NS_ERROR_FAILURE;
    },
    setTree: function(tree) {
	this._tree = tree;
    },
    toggleOpenState: function(index) { },

    // Alarm listener methods:
    alarmAdded: function(alarm) {
	this.syncView();
    },
    alarmRemoved: function(alarm) {
	this.syncView();
    },
    alarmFired: function(alarm) {
	// ignored, in favor of:
    },
    alarmsAltered: function() {
	this.syncView();
    }
};
