/*
 * test.js
 *
 * Copyright (c) 2005 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the Vana'diel Clock.
 *
 * The Vana'diel Clock is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The Vana'diel Clock is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Vana'diel Clock; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Runs several tests on the clock.
 */

const MINUTE = 60;
const HOUR = 60 * MINUTE;
const DAY = 24 * HOUR;
const WEEK = 8 * DAY;
const LUNAR_MONTH = DAY * 84;

var PATTERN_TESTS = [
	[ "0 0 0", 0, WEEK ],
	[ "0 5 0", 0, 5*HOUR ],
	[ "0 5 3", 0, 5*HOUR + 3*MINUTE],
	[ "* 5 0", 0, 5*HOUR ],
	[ "0 5,21 0", 6*HOUR, 21*HOUR ],
	[ "* 5,21 0", 6*HOUR, 21*HOUR ],
	[ "1 0 0", 0, DAY ],
	[ "0,1,2,3,4 0 0", 0, DAY ],
	[ "0,1,2,3,4 0 0", DAY, DAY*2 ],
	[ "0,1,2,3,4 0 0", DAY*2, DAY*3 ],
	[ "0,1,2,3,4 0 0", DAY*2+1500, DAY*3 ],
	[ "0,1,2,3,4 0 0", DAY*5, DAY*8 ],
	[ "* 0 0 0", 0, 12 * DAY ],
	// The following pattern is INVALID, as 38-44 isn't a valid moon phase
	// It should become * 5,21 0 11
	[ "* 5,21 0 38-44", 0, (7*11+12) * DAY + 5 * HOUR - LUNAR_MONTH],
	[ "* 5,21 0 11", 0, (7*11+12) * DAY + 5 * HOUR - LUNAR_MONTH]
];

localizeTimeFormatStrings(); // required to build the tests

var FORMAT_TESTS = [
	[ new Date(2004, 04, 10, 12, 0, 0), true, "12" + minSeparator + "00" + secSeparator + "00 " + earthPM ],
	[ new Date(2004, 04, 10, 12, 0, 0), false, "12" + minSeparator + "00 " + earthPM ]
];

var console_text = "";

function runTests() {
    console(" ==== STARTING TESTS ====\n");
    // start by creating a Vana'diel alarm.
    var alarm = Components.classes["@xenoveritas.org/vanadiel-alarm;1"].
		    createInstance(Components.interfaces.nsIVanadielAlarm);
    var msg = runTest("pattern-test-image", runPatternTests, alarm);
    msg += runTest("alarm-test-image", runAlarmTests, alarm);
    msg += runTest("moon-test-image", runMoonTests);
    msg += runTest("format-test-image", runFormatTests);
    msg += runTest("intsize-test-image", runIntSizeTest);
    document.getElementById("console").value = console_text;
}

function runTest(testName, testFunc, arg) {
    var test = "fail";
    var msg = "";
    try {
	test = testFunc(arg);
    } catch (ex) {
	console("***** TEST FAILED\n");
	console("  An exception was thrown!\n");
	console(ex);
	console("\n");
	for (var key in ex) {
	    console("exception.");
	    console(key);
	    console("=[");
	    console(ex[key]);
	    console("]\n");
	}
	msg = "An exception was thrown running tests!\n" +
		(ex.name ? "Exception " + ex.name : "The exception") +
		" was thrown in " + (ex.filename ? ex.filename : "an unknown file") +
		(ex.lineNumber != null ? ", line " + ex.lineNumber : " on an unknown line") +
		(ex.columnNumber != null ? ", column " + ex.columnNumber : " at an unknown column") +
		".\n" + (ex.message != null ? ex.message : "No detailed message was given.");
	if (ex.location) {
	    msg = msg + "\n" + ex.location;
	}
    }
    document.getElementById(testName).setAttribute("test", test);
    return msg;
}

function dumpTime(time) {
    var u = Math.floor(time / DAY);
    time -= u * DAY;
    var phase = (u + 26) % 84;
    console(u + " days, ");
    u = Math.floor(time / HOUR);
    time -= u * HOUR;
    console(u + ":");
    u = Math.floor(time / MINUTE);
    if (u < 10)
	console("0");
    console(u);
    console(", moon " + phase);
    if (phase >= 38)
	phase = Math.floor((phase - 38) / 7);
    else
	phase = Math.floor((phase + 46) / 7);
    console(", basic " + phase + "=" + moonPhases[phase]);
}

function runPatternTests(alarm) {
    var v;
    var pass = 0;
    var fail = 0;
    var rows = document.getElementById("pattern-test-rows");
    var row;
    var elem;
    console("Running pattern tests...\n");
    for (var t in PATTERN_TESTS) {
	row = document.createElement("row");
	rows.appendChild(row);
	console(" - Running pattern test on pattern \"");
	console(PATTERN_TESTS[t][0]);
	console("\" starting on ");
	console(PATTERN_TESTS[t][1]);
	console(", expecting ");
	console(PATTERN_TESTS[t][2]);
	alarm.setAlarm(PATTERN_TESTS[t][1], PATTERN_TESTS[t][0]);
	v = alarm.time;
	console(", got ");
	console(v);
	elem = document.createElement("image");
	elem.setAttribute("class", "test-image");
	if (v == PATTERN_TESTS[t][2]) {
	    console(" - passed.\n");
	    pass++;
	    elem.setAttribute("test", "success");
	} else {
	    console("(");
	    dumpTime(v);
	    console(") - FAILED.\n");
	    fail++;
	    console(alarm.debugMessage);
	    elem.setAttribute("test", "fail");
	}
	row.setAttribute("debugmsg", alarm.debugMessage);
	row.appendChild(elem);
	elem = document.createElement("label");
	elem.setAttribute("value", PATTERN_TESTS[t][0] + ", wanted " + PATTERN_TESTS[t][2] + ", got " + v);
	row.appendChild(elem);
    }
    console("Pattern tests complete, ");
    console(pass);
    console(" passed, ");
    console(fail);
    console(" failed.\n");
    if (fail == 0)
	return "success";
    if (pass == 0)
	return "fail";
    return "mixed";
}

function runAlarmTests(alarm) {
    var pass = 0;
    var fail = 0;
    console("Running alarm tests...\n");
    alarm.preAlertTime = 1500;
    alarm.time = DAY;
    var image = document.getElementById("alarm-pre-test-image");
    if (alarm.nextAlarm == DAY - 1500) {
	pass++;
	image.setAttribute("test", "success");
    } else {
	fail++;
	image.setAttribute("test", "fail");
    }
    image = document.getElementById("alarm-at-test-image");
    alarm.activate(DAY - 1500);
    if (alarm.nextAlarm == DAY) {
	pass++;
	image.setAttribute("test", "success");
    } else {
	console("Alarm at test failed, expected " + DAY + " but got " + alarm.nextAlarm + "\n");
	fail++;
	image.setAttribute("test", "fail");
    }
    image = document.getElementById("alarm-dead-test-image");
    alarm.activate(DAY);
    if (!alarm.active) {
	pass++;
	image.setAttribute("test", "success");
    } else {
	fail++;
	image.setAttribute("test", "fail");
    }
    console("Alarm tests complete, ");
    console(pass);
    console(" passed, ");
    console(fail);
    console(" failed.\n");
    if (fail == 0)
	return "success";
    if (pass == 0)
	return "fail";
    return "mixed";
}

function runMoonTests() {
    var pass = 0;
    var fail = 0;
    var rows = document.getElementById("moon-phase-test-rows");
    var row;
    var elem;
    // stop the clock, placing it into test mode
    VanadielClock_TimerService.stop();
    // run through
    console("Running moon tests...\n");
    var t, basic;
    var expected = 0;
    var expectDay = 0;
    var f = DAY * 12 + LUNAR_MONTH;
    for (t = DAY * 12; t <= f; t += DAY) {
	VanadielClock_TimerService.test(t);
	console("Time: ");
	console(t);
	console("; Day: ");
	console(VanadielClock_TimerService.day);
	console("; Moon phase: ");
	console(VanadielClock_TimerService.moonPhase);
	console(" (");
	console(VanadielClock_TimerService.moonPercent);
	console(" ");
	console(VanadielClock_TimerService.moonPhaseWaxing ? "waxing" : "waning");
	console("), basic: ");
	basic = VanadielClock_TimerService.basicMoonPhase;
	console(basic);
	row = document.createElement("row");
	rows.appendChild(row);
	elem = document.createElement("image");
	elem.setAttribute("class", "test-image");
	if (basic == expected && basic >= 0 && basic < moonPhases.length) {
	    console(" - passed.\n");
	    pass++;
	    elem.setAttribute("test", "success");
	} else {
	    console(" - FAILED.\n");
	    fail++;
	    elem.setAttribute("test", "fail");
	}
	row.appendChild(elem);
	elem = document.createElement("label");
	elem.setAttribute("value", "Epoch day " + VanadielClock_TimerService.epochDays + ", expect phase " + expected + ", got " + basic + ", phase: " + (basic < moonPhases.length ? moonPhases[basic] : "out of range"));
	row.appendChild(elem);
	expectDay++;
	if (expectDay >= 7) {
	    expectDay = 0;
	    expected++;
	    if (expected >= moonPhases.length)
		expected = 0;
	}
    }
    // restart the clock
    VanadielClock_TimerService.start();
    console("Time tests complete, ");
    console(pass);
    console(" passed, ");
    console(fail);
    console(" failed.\n");
    if (fail == 0)
	return "success";
    if (pass == 0)
	return "fail";
    return "mixed";
}

function runFormatTests() {
    var pass = 0;
    var fail = 0;
    var rows = document.getElementById("format-test-rows");
    var row, elem, got;
    console("Running date format tests...\n");
    for (var t in FORMAT_TESTS) {
	var got = formatEarthTime(FORMAT_TESTS[t][0], FORMAT_TESTS[t][1]);
	console(" - Format " + FORMAT_TESTS[t][0] + ", got \"" + got + "\"");
	row = document.createElement("row");
	rows.appendChild(row);
	elem = document.createElement("image");
	elem.setAttribute("class", "test-image");
	if (got == FORMAT_TESTS[t][2]) {
	    pass++;
	    console(", passed.\n");
	    elem.setAttribute("test", "success");
	} else {
	    row.setAttribute("debugmsg", "Wanted \"" + FORMAT_TESTS[t][2] + "\" but got \"" + got + "\".");
	    fail++;
	    console(", expected \"" + FORMAT_TESTS[t][2] + "\" - FAILED.\n");
	    elem.setAttribute("test", "fail");
	}
	row.appendChild(elem);
	elem = document.createElement("label");
	elem.setAttribute("value", "Format [" + FORMAT_TESTS[t][0] + "] to \"" + got + "\"");
	row.appendChild(elem);
    }
    console("Format tests complete, ");
    console(pass);
    console(" passed, ");
    console(fail);
    console(" failed.\n");
    if (fail == 0)
	return "success";
    if (pass == 0)
	return "fail";
    return "mixed";
}

function runIntSizeTest() {
    console("Running integer size test...\n");
    var pass = 0;
    var fail = 0;
    var i = 1;
    for (var s = 2; s <= 64; s++) {
	i <<= 1;
	console("Shift " + s + ", value: " + i + "\n");
	if (i < 0) {
	    console("Value wrapped, max int size determined.\n");
	    document.getElementById("max-int-size").value = s + " bits";
	    document.getElementById("max-int-size-image").setAttribute("test", "mixed");
	    break;
	}
    }
    if (i > 0) {
	document.getElementById("max-int-size").value = "at least " + s + " bits";
	document.getElementById("max-int-size-image").setAttribute("test", "success");
	pass++;
    }
    // Attempt to take 2^31-1 and add 1 to it
    i = 0x7FFFFFFF;
    i++;
    document.getElementById("int-auto-promote").value = i;
    document.getElementById("int-auto-promote-image").setAttribute("test", i > 0 ? "success" : "fail");
    if (i > 0) {
	pass++;
    } else {
	fail++;
    }
    if (fail == 0)
	return "success";
    if (pass == 0)
	return "fail";
    return "mixed";
}

function console(msg) {
    @DEBUG@ dump(msg);
    console_text = console_text + msg;
}

function consoleMessage(event) {
    var msg = "No message.";
    var n = event.target;
    while (n != null) {
	if (n.nodeType == document.ELEMENT_NODE && n.hasAttribute("debugmsg")) {
	    msg = n.getAttribute("debugmsg");
	    break;
	}
	n = n.parentNode;
    }
    document.getElementById("testmessage").value = msg;
    return true;
}
